@@ -1,12 +1,18 @@
 import bisect
-import gobject
-import orca_state
-import pyatspi
-import speech
 import copy
+import pyatspi
 import time
+from gi.repository import GLib
 
-from orca_i18n import _
+from . import cmdnames
+from . import chnames
+from . import keybindings
+from . import messages
+from . import input_event
+from . import orca_state
+from . import settings_manager
+
+_settingsManager = settings_manager.getManager()
 
 # define 'live' property types
 LIVE_OFF       = -1
@@ -50,55 +56,14 @@ class PriorityQueue:
         time """
         currenttime = time.time()
         myfilter = lambda item: item[1] + MSG_KEEPALIVE_TIME > currenttime
-        self.queue = filter(myfilter, self.queue)
+        self.queue = list(filter(myfilter, self.queue))
 
     def purgeByPriority(self, priority):
         """ Purge items from the queue that have a lower than or equal priority
         than the given argument """
         myfilter = lambda item: item[0] > priority
-        self.queue = filter(myfilter, self.queue)
+        self.queue = list(filter(myfilter, self.queue))
 
-    def clumpContents(self):
-        """ Combines messages with the same 'label' by appending newer  
-        'content' and removing the newer message.  This operation is only
-        applied to the next dequeued message for performance reasons and is
-        often applied in conjunction with filterContents() """
-        if len(self.queue):
-            newqueue = []
-            newqueue.append(self.queue[0])
-            targetlabels = newqueue[0][2]['labels']
-            targetcontent = newqueue[0][2]['content']
-            for i in range(1, len(self.queue)):
-                if self.queue[i][2]['labels'] == targetlabels:
-                    newqueue[0][2]['content'].extend \
-                                   (self.queue[i][2]['content'])
-                else:
-                    newqueue.append(self.queue[i]) 
-
-            self.queue = newqueue
-
-    def filterContents(self):
-        """ Combines utterances by eliminating repeated utterances and
-        utterances that are part of other utterances. """
-        if len(self.queue[0][2]['content']) > 1:
-            oldcontent = self.queue[0][2]['content']
-            newcontent = [oldcontent[0]]
-
-            for i in range(1, len(oldcontent)):
-                found = False
-                for j in range(len(newcontent)):
-                    if oldcontent[i].find(newcontent[j]) != -1 \
-                        or newcontent[j].find(oldcontent[i]) != -1: 
-                        if len(oldcontent[i]) > len(newcontent[j]):
-                            newcontent[j] = oldcontent[i]
-                        found = True
-                        break
-
-                if not found:
-                    newcontent.append(oldcontent[i])
-
-            self.queue[0][2]['content'] = newcontent
- 
     def __len__(self):
         """ Return the length of the queue """
         return len(self.queue)
@@ -110,6 +75,15 @@ class LiveRegionManager:
         # message priority queue
         self.msg_queue = PriorityQueue()
 
+        self.inputEventHandlers = self._getInputEventHandlers()
+        self.keyBindings = self._getKeyBindings()
+
+        # This is temporary.
+        self.functions = [self.advancePoliteness,
+                          self.setLivePolitenessOff,
+                          self.toggleMonitoring,
+                          self.reviewLiveAnnouncement]
+
         # Message cache.  Used to store up to 9 previous messages so user can
         # review if desired.
         self.msg_cache = []
@@ -137,12 +111,71 @@ class LiveRegionManager:
         script.bookmarks.addSaveObserver(self.bookmarkSaveHandler)
         script.bookmarks.addLoadObserver(self.bookmarkLoadHandler)
 
+    def _getInputEventHandlers(self):
+        handlers = {}
+
+        handlers["advanceLivePoliteness"] = \
+            input_event.InputEventHandler(
+                self.advancePoliteness,
+                cmdnames.LIVE_REGIONS_ADVANCE_POLITENESS)
+
+        handlers["setLivePolitenessOff"] = \
+            input_event.InputEventHandler(
+                self.setLivePolitenessOff,
+                cmdnames.LIVE_REGIONS_SET_POLITENESS_OFF)
+
+        handlers["monitorLiveRegions"] = \
+            input_event.InputEventHandler(
+                self.toggleMonitoring,
+                cmdnames.LIVE_REGIONS_MONITOR)
+
+        handlers["reviewLiveAnnouncement"] = \
+            input_event.InputEventHandler(
+                self.reviewLiveAnnouncement,
+                cmdnames.LIVE_REGIONS_REVIEW)
+
+        return handlers
+
+    def _getKeyBindings(self):
+        keyBindings = keybindings.KeyBindings()
+
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "backslash",
+                keybindings.defaultModifierMask,
+                keybindings.NO_MODIFIER_MASK,
+                self.inputEventHandlers.get("advanceLivePoliteness")))
+
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "backslash",
+                keybindings.defaultModifierMask,
+                keybindings.SHIFT_MODIFIER_MASK,
+                self.inputEventHandlers.get("setLivePolitenessOff")))
+
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "backslash",
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_SHIFT_MODIFIER_MASK,
+                self.inputEventHandlers.get("monitorLiveRegions")))
+
+        for key in ["F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9"]:
+            keyBindings.add(
+                keybindings.KeyBinding(
+                    key,
+                    keybindings.defaultModifierMask,
+                    keybindings.ORCA_MODIFIER_MASK,
+                    self.inputEventHandlers.get("reviewLiveAnnouncement")))
+
+        return keyBindings
+
     def reset(self):
         # First we will purge our politeness override dictionary of LIVE_NONE
         # objects that are not registered for this page
         newpoliteness = {}
         currenturi = self._script.bookmarks.getURIKey()
-        for key, value in self._politenessOverrides.iteritems():
+        for key, value in self._politenessOverrides.items():
             if key[0] == currenturi or value != LIVE_NONE:
                 newpoliteness[key] = value
         self._politenessOverrides = newpoliteness
@@ -182,7 +215,7 @@ class LiveRegionManager:
         message = self._getMessage(event)
         if message:
             if len(self.msg_queue) == 0:
-                gobject.timeout_add(100, self.pumpMessages)
+                GLib.timeout_add(100, self.pumpMessages)
             self.msg_queue.enqueue(message, politeness, event.source)
 
     def pumpMessages(self):
@@ -190,23 +223,9 @@ class LiveRegionManager:
         purging the message queue and outputting any queued messages that
         were queued up in the handleEvent() method.
         """
-        # If there are messages in the queue, we are monitoring, and we are not
-        # currently speaking then speak queued message.
-        # Note: Do all additional work within if statement to prevent
-        # it from being done for each event loop callback
-        # Note: isSpeaking() returns False way too early.  A strategy using
-        # a message length (in secs) could be used but don't forget many 
-        # parameters such as rate,expanded text and others must be considered.
-        if len(self.msg_queue) > 0 \
-                  and not speech.isSpeaking() \
-                  and time.time() - orca_state.lastInputEvent.time > 1:
-            # House cleaning on the message queue.  
-            # First we will purge the queue of old messages
+
+        if len(self.msg_queue) > 0:
             self.msg_queue.purgeByKeepAlive()
-            # Next, we will filter the messages
-            self.msg_queue.clumpContents()
-            self.msg_queue.filterContents()
-            # Let's get our queued information
             politeness, timestamp, message, obj = self.msg_queue.dequeue()
             # Form output message.  No need to repeat labels and content.
             # TODO: really needs to be tested in real life cases.  Perhaps
@@ -215,7 +234,7 @@ class LiveRegionManager:
                 utts = message['content']
             else:
                 utts = message['labels'] + message['content']
-            speech.speakUtterances(utts)
+            self._script.presentMessage(utts)
 
             # set the last live obj to be announced
             self.lastliveobj = obj
@@ -228,11 +247,8 @@ class LiveRegionManager:
             self.msg_queue.purgeByKeepAlive()
 
         # See you again soon, stay in event loop if we still have messages.
-        if len(self.msg_queue) > 0:
-            return True 
-        else:
-            return False
-        
+        return len(self.msg_queue) > 0
+
     def getLiveNoneObjects(self):
         """Return the live objects that are registered and have a politeness
         of LIVE_NONE. """
@@ -243,9 +259,14 @@ class LiveRegionManager:
                 retval.append(self._script.bookmarks.pathToObj(objectid))
         return retval
 
-    def advancePoliteness(self, obj):
+    def advancePoliteness(self, script, inputEvent):
         """Advance the politeness level of the given object"""
-        utterances = []
+
+        if not _settingsManager.getSetting('inferLiveRegions'):
+            self._script.presentMessage(messages.LIVE_REGIONS_OFF)
+            return
+
+        obj = orca_state.locusOfFocus
         objectid = self._getObjectId(obj)
         uri = self._script.bookmarks.getURIKey()
 
@@ -260,61 +281,56 @@ class LiveRegionManager:
 
         if cur_priority == LIVE_OFF or cur_priority == LIVE_NONE:
             self._politenessOverrides[(uri, objectid)] = LIVE_POLITE
-            # Translators:  sets the live region politeness level to polite
-            #
-            utterances.append(_('setting live region to polite'))
+            self._script.presentMessage(messages.LIVE_REGIONS_LEVEL_POLITE)
         elif cur_priority == LIVE_POLITE:
             self._politenessOverrides[(uri, objectid)] = LIVE_ASSERTIVE
-            # Translators:  sets the live region politeness level to assertive
-            #
-            utterances.append(_('setting live region to assertive'))
+            self._script.presentMessage(messages.LIVE_REGIONS_LEVEL_ASSERTIVE)
         elif cur_priority == LIVE_ASSERTIVE:
             self._politenessOverrides[(uri, objectid)] = LIVE_RUDE
-            # Translators:  sets the live region politeness level to rude
-            #
-            utterances.append(_('setting live region to rude'))
+            self._script.presentMessage(messages.LIVE_REGIONS_LEVEL_RUDE)
         elif cur_priority == LIVE_RUDE:
             self._politenessOverrides[(uri, objectid)] = LIVE_OFF
-            # Translators:  sets the live region politeness level to off
-            #
-            utterances.append(_('setting live region to off'))
+            self._script.presentMessage(messages.LIVE_REGIONS_LEVEL_OFF)
 
-        speech.speakUtterances(utterances)
 
     def goLastLiveRegion(self):
         """Move the caret to the last announced live region and speak the 
         contents of that object"""
         if self.lastliveobj:
-            self._script.setCaretPosition(self.lastliveobj, 0)
-            self._script.outlineAccessible(self.lastliveobj)
-            self._script.speakContents(self._script.getObjectContentsAtOffset(\
-                                       self.lastliveobj,0))
+            self._script.utilities.setCaretPosition(self.lastliveobj, 0)
+            self._script.speakContents(self._script.utilities.getObjectContentsAtOffset(
+                                       self.lastliveobj, 0))
 
-    def reviewLiveAnnouncement(self, msgnum):
+    def reviewLiveAnnouncement(self, script, inputEvent):
         """Speak the given number cached message"""
-        if msgnum > len(self.msg_cache):
-            # Tranlators: this tells the user that a cached message
-            # is not available.
-            #
-            speech.speak(_('no live message saved'))
-        else:
-            speech.speakUtterances(self.msg_cache[-msgnum])
 
-    def setLivePolitenessOff(self):
+        msgnum = int(inputEvent.event_string[1:])
+        if not _settingsManager.getSetting('inferLiveRegions'):
+            self._script.presentMessage(messages.LIVE_REGIONS_OFF)
+            return
+
+        if msgnum > len(self.msg_cache):
+            self._script.presentMessage(messages.LIVE_REGIONS_NO_MESSAGE)
+        else:
+            self._script.presentMessage(self.msg_cache[-msgnum])
+
+    def setLivePolitenessOff(self, script, inputEvent):
         """User toggle to set all live regions to LIVE_OFF or back to their
         original politeness."""
+
+        if not _settingsManager.getSetting('inferLiveRegions'):
+            self._script.presentMessage(messages.LIVE_REGIONS_OFF)
+            return
+
         # start at the document frame
-        docframe = self._script.getDocumentFrame()
+        docframe = self._script.utilities.documentFrame()
         # get the URI of the page.  It is used as a partial key.
         uri = self._script.bookmarks.getURIKey()
 
         # The user is currently monitoring live regions but now wants to 
         # change all live region politeness on page to LIVE_OFF
         if self.monitoring:
-            # Translators: This lets the user know that all live regions
-            # have been turned off.
-            speech.speak(_("All live regions set to off"))
-
+            self._script.presentMessage(messages.LIVE_REGIONS_ALL_OFF)
             self.msg_queue.clear()
             
             # First we'll save off a copy for quick restoration
@@ -337,22 +353,19 @@ class LiveRegionManager:
 
         # The user wants to restore politeness levels
         else:
-            for key, value in self._restoreOverrides.iteritems():
+            for key, value in self._restoreOverrides.items():
                 self._politenessOverrides[key] = value
-            # Translators: This lets the user know that all live regions
-            # have been restored to their original politeness level.
-            speech.speak(_("live regions politeness levels restored"))
-
+            self._script.presentMessage(messages.LIVE_REGIONS_ALL_RESTORED)
             # Toggle our flag
             self.monitoring = True  
 
-    def outputLiveRegionDescription(self, obj):
-        """Used in conjuction with whereAmI to output description and 
+    def generateLiveRegionDescription(self, obj, **args):
+        """Used in conjunction with whereAmI to output description and 
         politeness of the given live region object"""
         objectid = self._getObjectId(obj)
         uri = self._script.bookmarks.getURIKey()
 
-        utterances = []
+        results = []
 
         # get the description if there is one.
         for relation in obj.getRelationSet():
@@ -367,7 +380,7 @@ class LiveRegionManager:
                     #
                     description = targetobj.queryText().getText(0, -1)
                     if description.strip() != obj.description.strip():
-                        utterances.append(description)
+                        results.append(description)
                 except NotImplemented:
                     pass
 
@@ -380,11 +393,10 @@ class LiveRegionManager:
 
         # We will only output useful information
         # 
-        if utterances or liveprioritystr != 'none':
-            # Translators: output the politeness level
-            #
-            utterances.append(_('politeness level %s') %liveprioritystr)
-            speech.speakUtterances(utterances)
+        if results or liveprioritystr != 'none':
+            results.append(messages.LIVE_REGIONS_LEVEL % liveprioritystr)
+
+        return results
 
     def matchLiveRegion(self, obj):
         """Predicate used to find a live region"""
@@ -394,76 +406,48 @@ class LiveRegionManager:
     def _getMessage(self, event):
         """Gets the message associated with a given live event."""
         attrs = self._getAttrDictionary(event.source)
-        content = [] 
-        labels = []
+        content = ""
+        labels = ""
         
         # A message is divided into two parts: labels and content.  We
         # will first try to get the content.  If there is None, 
         # assume it is an invalid message and return None
         if event.type.startswith('object:children-changed:add'):
-            # Get the text based on the atomic property
-            try:
-                if attrs['container-atomic'] == 'true':
-                    # expand the source if atomic is true
-                    newcontent = self._script.expandEOCs(event.source)
-                else:
-                    # expand the target if atomic is false
-                    newcontent = self._script.expandEOCs(event.any_data)
-            except (KeyError, TypeError):
-                # expand the target if there is no ARIA markup
-                newcontent = self._script.expandEOCs(event.any_data)
-
-            # add our content to the returned message or return None if no
-            # content
-            if newcontent:
-                content.append(newcontent)
+            if attrs.get('container-atomic') == 'true':
+                content = self._script.utilities.expandEOCs(event.source)
             else:
-                return None
+                content = self._script.utilities.expandEOCs(event.any_data)
 
-        else: #object:text-changed:insert
-            # Get a handle to the Text interface for the source.
-            # Serious problems if this fails.
-            #
-            try:
-                sourceitext = event.source.queryText()
-            except NotImplementedError:
-                return None
-
-            # We found an embed character.  We can expect a children-changed
-            # event, which we will act on, so just return.
-            txt = sourceitext.getText(0, -1)
-            if txt.count(self._script.EMBEDDED_OBJECT_CHARACTER) > 0:
-                return None
-
-            # Get the text based on the atomic property
-            try:
-                if attrs['container-atomic'] == 'true':
-                    newcontent = txt
-                else:
-                    newcontent = txt[event.detail1:event.detail1+event.detail2]
-            except KeyError:
-                newcontent = txt[event.detail1:event.detail1+event.detail2]
-
-            # add our content to the returned message or return None if no
-            # content
-            if len(newcontent) > 0:
-                content.append(newcontent)
+        elif event.type.startswith('object:text-changed:insert'):
+            if attrs.get('container-atomic') != 'true':
+                content = event.any_data
             else:
-                return None
+                text = self._script.utilities.queryNonEmptyText(event.source)
+                if text:
+                    content = text.getText(0, -1)
 
-        # Get the labeling information now that we have good content.
-        labels = self._getLabelsAsUtterances(event.source)
+        if not content:
+            return None
+
+        content = content.strip()
+        if len(content) == 1:
+            content = chnames.getCharacterName(content)
+
+        # Proper live regions typically come with proper aria labels. These
+        # labels are typically exposed as names. Failing that, descriptions.
+        # Looking for actual labels seems a non-performant waste of time.
+        name = (event.source.name or event.source.description).strip()
+        if name and name != content:
+            labels = name
 
         # instantly send out notify messages
-        if 'channel' in attrs and attrs['channel'] == 'notify':
+        if attrs.get('channel') == 'notify':
             utts = labels + content
-            speech.stop()
-            # Note: we would like to use a different ACSS for alerts.  This work
-            # should be done as part of bug #412656.
-            speech.speakUtterances(utts)
+            self._script.presentationInterrupt()
+            self._script.presentMessage(utts)
             return None
-        else:
-            return {'content':content, 'labels':labels}
+
+        return {'content':[content], 'labels':[labels]}
 
     def flushMessages(self):
         self.msg_queue.clear()
@@ -474,42 +458,6 @@ class LiveRegionManager:
         if len(self.msg_cache) > CACHE_SIZE:
             self.msg_cache.pop(0)
 
-    def _getLabelsAsUtterances(self, obj):
-        """Get the labels for a given object"""
-        # try the Gecko label getter first
-        uttstring = self._script.getDisplayedLabel(obj)
-        if uttstring:
-            return [uttstring.strip()]
-        # often we see a table cell.  I'll implement my own label getter
-        elif obj.getRole() == pyatspi.ROLE_TABLE_CELL \
-                           and obj.parent.childCount > 1:
-            # We will try the table interface first for it's parent
-            try:
-                itable = obj.parent.queryTable()
-                # I'm in a table, now what row are we in?  Look in the first 
-                # columm of that row.
-                #
-                # Note: getRowHeader() fails for most markup.  We will use the
-                # relation when the markup is good (when getRowHeader() works) 
-                # so we won't see this code in those cases.  
-                index = self._script.getCellIndex(obj)
-                row = itable.getRowAtIndex(index)
-                header = itable.getAccessibleAt(row, 0)
-                # expand the header
-                return [self._script.expandEOCs(header).strip()]
-            except NotImplementedError:
-                pass
-
-            # Last ditch effort is to see if our parent is a table row <tr> 
-            # element.
-            parentattrs = self._getAttrDictionary(obj.parent) 
-            if 'tag' in parentattrs and parentattrs['tag'] == 'TR':
-                return [self._script.expandEOCs( \
-                                  obj.parent.getChildAtIndex(0)).strip()]
-
-        # Sorry, no valid labels found
-        return []
-
     def _getLiveType(self, obj):
         """Returns the live politeness setting for a given object. Also,
         registers LIVE_NONE objects in politeness overrides when monitoring."""
@@ -577,9 +525,9 @@ class LiveRegionManager:
     def _getPath(self, obj):
         """ Returns, as a tuple of integers, the path from the given object 
         to the document frame."""
-        docframe = self._script.getDocumentFrame()
+        docframe = self._script.utilities.documentFrame()
         path = []
-        while 1:
+        while True:
             if obj.parent is None or obj == docframe:
                 path.reverse()
                 return tuple(path)
@@ -588,3 +536,12 @@ class LiveRegionManager:
             except Exception:
                 raise LookupError
             obj = obj.parent
+
+    def toggleMonitoring(self, script, inputEvent):
+        if not _settingsManager.getSetting('inferLiveRegions'):
+            _settingsManager.setSetting('inferLiveRegions', True)
+            self._script.presentMessage(messages.LIVE_REGIONS_MONITORING_ON)
+        else:
+            _settingsManager.setSetting('inferLiveRegions', False)
+            self.flushMessages()
+            self._script.presentMessage(messages.LIVE_REGIONS_MONITORING_OFF)
