@@ -55,7 +55,6 @@ from . import settings_manager
 from . import sound_generator
 from . import speech_generator
 from . import structural_navigation
-from . import where_am_I
 from . import bookmarks
 from . import tutorialgenerator
 
@@ -115,7 +114,6 @@ class Script:
         self.speechGenerator = self.getSpeechGenerator()
         self.generatorCache = {}
         self.eventCache = {}
-        self.whereAmI = self.getWhereAmI()
         self.spellcheck = self.getSpellCheck()
         self.tutorialGenerator = self.getTutorialGenerator()
 
@@ -155,25 +153,6 @@ class Script:
 
         return keybindings.KeyBindings()
 
-    def getKeyBindingsForInputHandler(self, inputEventHandler):
-        """ Returns a KeyBindings object with the list of KeyBindings that
-        matche the passed inputEventHandler as argument (at least the
-        inputEventHandler that has the same handler function)
-
-        Arguments:
-        - inputEventHandler: an instance of input_event.InputEventHandler
-
-        Returns an instance of keybindings.KeyBindings populated with
-        keybindings.KeyBinding instances that match the inputEventHandler.
-        """
-        matches = keybindings.KeyBindings()
-
-        for binding in self.keyBindings.keyBindings:
-            if inputEventHandler == binding.handler:
-                matches.add(binding)
-
-        return matches
-
     def getBrailleBindings(self):
         """Defines the braille bindings for this script.
 
@@ -182,20 +161,6 @@ class Script:
         """
         return {}
 
-    def getBrailleCommandsForInputHandler(self, inputEventHandler):
-        """Returns a list of BrlTTY commands (they're in braille.py) that
-        match the given inputEventHandler passed as argument.
-
-        Arguments:
-        - inputEventHandler: an instance of input_event.InputEventHandler
-
-        Returns a list (possibly empty) of BrlTTY commands (they're in
-        braille.py) that match the given inputEventHandler passed.
-        """
-        return [command
-                for command, handler in self.brailleBindings.items()
-                if inputEventHandler == handler]
-
     def getFormatting(self):
         """Returns the formatting strings for this script."""
         return formatting.Formatting(self)
@@ -266,11 +231,6 @@ class Script:
         """
         return self.structuralNavigation.enabled
 
-    def getWhereAmI(self):
-        """Returns the "where am I" class for this script.
-        """
-        return where_am_I.WhereAmI(self)
-
     def getBookmarks(self):
         """Returns the "bookmarks" class for this script.
         """
@@ -566,24 +526,6 @@ class Script:
         """
         pass
 
-    def visualAppearanceChanged(self, event, obj):
-        """Called when the visual appearance of an object changes.
-        This method should not be called for objects whose visual
-        appearance changes solely because of focus -- setLocusOfFocus
-        is used for that.  Instead, it is intended mostly for objects
-        whose notional 'value' has changed, such as a checkbox
-        changing state, a progress bar advancing, a slider moving,
-        text inserted, caret moved, etc.
-
-        The primary purpose of this method is to present the changed
-        information to the user.
-
-        Arguments:
-        - event: if not None, the Event that caused this to happen
-        - obj: the Accessible whose visual appearance changed.
-        """
-        pass
-
     def isActivatableEvent(self, event):
         """Returns True if the given event is one that should cause this
         script to become the active script.  This is only a hint to
@@ -602,3 +544,6 @@ class Script:
     def deactivate(self):
         """Called when this script is deactivated."""
         pass
+
+    def getTransferableAttributes(self):
+        return {}
